#include <dos.h>

#include "coml460.h"

/*
 *      func :  set communication parameters
 *
 *      input :
 *              portno  : port number   0 .. 64
 *              baud    : baud rate     600, 1200, 9600, 38400, 56000, 115200, 230400, 460800
 *              db      : data bits     5, 6, 7, 8
 *              parity  : parity bit    0 : nonparity, 1 : odd parity, 3 : even parity
 *              sb      : stop bits     0 : 1 bit, 1 : 2 bits
 *      output :
 *              sts     : port status
 */
int cominit(int portno, long baud, int db, int parity, int sb)
{
   unsigned char cmd = 0x00;
   union         REGS reg;

   switch (baud) {
      case B460800 :
	 cmd |= 0x20;
	 break;
      case B230400 :
	 cmd |= 0x00;
	 break;
      case  B115200 :
	 cmd |= 0xE0;
	 break;
      case  B56000 :
	 cmd |= 0xC0;
	 break;
      case  B38400 :
	 cmd |= 0xA0;
	 break;
      case  B9600 :
	 cmd |= 0x80;
	 break;
      case   B1200 :
	 cmd |= 0x60;
	 break;
      case   B600 :
	 cmd |= 0x40;
	 break;
      default :
	 return -1;
   }

   switch (parity) {
      case PNONE :              //non parity
	 cmd |= 0x00;
	 break;
      case PODD :               //odd parity
	 cmd |= 0x08;
	 break;
      case PEVEN :              //even parity
         cmd |= 0x0C;
         break;
      default :
         return -1;
   }

   switch (db) {
      case CS5 :
         cmd |= 0x00;
         break;
      case CS6 :
         cmd |= 0x01;
         break;
      case CS7 :
         cmd |= 0x02;
         break;
      case CS8 :
         cmd |= 0x03;
         break;
      default :
         return -1;
   }

   switch (sb) {
      case ST1 :
         cmd |= 0x00;
         break;
      case ST2 :
         cmd |= 0x04;
         break;
      default :
         return -1;
   }

   reg.x.dx = portno;
   reg.h.ah = COMINIT;
   reg.h.al = cmd;
   int86(0x14, &reg, &reg);

   return reg.x.ax;
}

/*
 *      func : Transmit character and wait
 *
 *      input :
 *              portno  : port number
 *              c       : character transmitted
 *      output :
 *              sts     : port status
 */
int comtxch(int portno, char c)
{
   union REGS reg;

   reg.x.dx = portno;
   reg.h.ah = COMSEND;
   reg.h.al = c;
   int86(0x14, &reg, &reg);

   return reg.x.ax;
}

/*
 *      func : Get received character with wait
 *
 *      input :
 *              portno  : port number
 *      output :
 *              c       : input character
 *              sts     : port status
 */
int comrxch(int portno, char *c)
{
  union REGS reg;

   reg.x.dx = portno;
   reg.h.ah = COMRECV;
   int86(0x14, &reg, &reg);
   *c = reg.h.al;

   return reg.h.ah;
}

/*
 *      func : return serial port status
 *
 *      input :
 *              portno  : port number
 *      output :
 *              sts     : port status
 */
int comstat(int portno)
{
   union REGS reg;

   reg.x.dx = portno;
   reg.h.ah = COMSTAT;
   int86(0x14, &reg, &reg);

   return reg.x.ax;
}

int comactive(int portno)
{
   union REGS reg;

   reg.x.dx = portno;
   reg.h.ah = COMACTIVE;
   int86(0x14, &reg, &reg);

   return reg.x.ax == 0x1954;
}

void comsetDTR(int portno, int dtr)
{
   union REGS reg;

   reg.x.dx = portno;
   reg.h.ah = COMSETDTR;
   reg.h.al = dtr;
   int86(0x14, &reg, &reg);
}

void compurgerxbuf(int portno)
{
   union REGS reg;

   reg.x.dx = portno;
   reg.h.ah = COMPURGERXBUF;
   int86(0x14, &reg, &reg);
}

int compeek(int portno)
{
   union REGS reg;

   reg.x.dx = portno;
   reg.h.ah = COMPEEK;
   int86(0x14, &reg, &reg);

   return reg.x.ax;
}

void comsetflow(int portno, int flow)
{
   union REGS reg;

   reg.x.dx = portno;
   reg.h.ah = COMSETFLOW;
   reg.h.al = flow;
   int86(0x14, &reg, &reg);
}

int comrxblk(int portno, void *ptr, int size)
{
   union REGS   reg;
   struct SREGS sreg;

   reg.x.dx = portno;
   reg.h.ah = COMRXBLK;
   reg.x.cx = size;
#if defined (__COMPACT__) || defined (__LARGE__) || defined (__HUGE__)
   reg.x.di = FP_OFF (ptr);
   sreg.es = FP_SEG (ptr);
#else
   reg.x.di = (int) ptr;
   sreg.es = _DS;
#endif
   int86x(0x14, &reg, &reg, &sreg);

   return reg.x.ax;
}

int comtxblk(int portno, void *ptr, int size)
{
   union REGS   reg;
   struct SREGS sreg;

   reg.x.dx = portno;
   reg.h.ah = COMTXBLK;
   reg.x.cx = size;
#if defined (__COMPACT__) || defined (__LARGE__) || defined (__HUGE__)
   reg.x.di = FP_OFF(ptr);
   sreg.es = FP_SEG(ptr);
#else
   reg.x.di = (int) ptr;
   sreg.es = _DS;
#endif
   int86x(0x14, &reg, &reg, &sreg);

   return reg.x.ax;
}

void combreak(int portno, int brk)
{
   union REGS reg;

   reg.x.dx = portno;
   reg.h.ah = COMSETBREAK;
   reg.h.al = brk;
   int86(0x14, &reg, &reg);
}

int cominstalled(int portno)
{
   union REGS                reg;
   struct SREGS              sreg;
   static struct FOSSIL_INFO info;

   info.majver = 0;
   reg.x.dx = portno;
   reg.h.ah = COMINFO;
   reg.x.cx = sizeof (info);
#if defined (__COMPACT__) || defined (__LARGE__) || defined (__HUGE__)
   reg.x.di = FP_OFF(&info);
   sreg.es = FP_SEG(&info);
#else
   reg.x.di = (int) &info;
   sreg.es = _DS;
#endif
   int86x(0x14, &reg, &reg, &sreg);

   return info.majver >= 5;
}

int comtxs(int portno, char *s)
{
   int sts = 0;

   while (*s) sts |= comtxch (portno, *s++);

   return sts;
}

int comrxs(int portno, char *s)
{
   int t;

   s--;
   do {
      t = comrxch (portno, ++s);
   } while (t == 0 && *s != 13);
   *s = 0;

   return t;
}

int combufstat(int portno)
{
   union REGS reg;

   reg.x.dx = portno;
   reg.h.ah = COMBUFSTAT;
   int86(0x14, &reg, &reg);

   return reg.x.cx;
}

int comtxbufstat(int portno)
{
   union REGS reg;

   reg.x.dx = portno;
   reg.h.ah = COMBUFSTAT;
   int86(0x14, &reg, &reg);

   return reg.x.dx;
}

void comsetport(int portno, int addr)
{
   union REGS reg;

   reg.x.dx = portno;
   reg.h.ah = COMSETPORT;
   reg.x.bx = addr;

   int86(0x14, &reg, &reg);
}

void comsetvector(int portno, int addr)
{
   union REGS reg;

   reg.x.dx = portno;
   reg.h.ah = COMSETVECTOR;
   reg.x.bx = addr;
   int86(0x14, &reg, &reg);
}

void comsetactive(int portno, char active)
{
   union REGS reg;

   reg.x.dx = portno;
   reg.h.ah = COMSETACTIVE;
   reg.h.al = active;
   int86(0x14, &reg, &reg);
}

void comsetMCR(int portno, int mcr)
{
   union REGS reg;

   reg.x.dx = portno;
   reg.h.ah = COMSETMCR;
   reg.h.al = mcr;
   int86(0x14, &reg, &reg);
}

char comgetparam(int portno)
{
   union REGS reg;

   reg.x.dx = portno;
   reg.h.ah = COMGETPARAM;
   int86(0x14, &reg, &reg);

   return reg.h.al;
}

void comgetinfo(int portno, long *baud, int *db, int *parity, int *sb)
{
   unsigned char r, n;
   const long b[8] = {
      B230400, B460800, B600, B1200, B9600, B38400, B56000, B115200
   };
   const int d[4] = {
      CS5, CS6, CS7, CS8
   };
   const int s[2] = {
      ST1, ST2
   };
   const int p[4] = {
      PNONE, PODD, PNONE, PEVEN
   };

   r = comgetparam(portno);
   n = (r >> 5) & 7;
   *baud = b[n];
   n = (r >> 3) & 3;
   *parity = p[n];
   n = (r >> 2) & 1;
   *sb = s[n];
   n = r & 3;
   *db = d[n];
}

char comgetMCR(int portno)
{
   union REGS reg;

   reg.x.dx = portno;
   reg.h.ah = COMGETMCR;
   int86(0x14, &reg, &reg);

   return reg.h.al;
}

char comgetFC(int portno)
{
   union REGS reg;

   reg.x.dx = portno;
   reg.h.ah = COMGETFC;
   int86(0x14, &reg, &reg);

   return reg.h.al;
}

int comgetsets(int portno)
{
   union REGS reg;

   reg.h.ah = COMGETSETS;
   reg.x.dx = portno;
   int86(0x14, &reg, &reg);

   return reg.x.ax;
}

BiosComSet_t *comgetsetinfo(int portno)
{
   union REGS          reg;
   struct SREGS        sreg;
   static BiosComSet_t info;

   reg.x.dx = portno;
   reg.h.ah = COMGETSETINFO;
#if defined (__COMPACT__) || defined (__LARGE__) || defined (__HUGE__)
   reg.x.di = FP_OFF(&info);
   sreg.es = FP_SEG(&info);
#else
   reg.x.di = (int) &info;
   sreg.es = _DS;
#endif
   int86x(0x14, &reg, &reg, &sreg);

   return &info;
}

int comgetports(int portno)
{
   union REGS reg;

   reg.h.ah = COMGETPORTS;
   reg.x.dx = portno;
   int86(0x14, &reg, &reg);

   return reg.x.ax;
}

int comdetectport(int portno)
{
   union REGS           reg;

   reg.x.dx = portno;
   reg.h.ah = COMDETECTPORT;
   int86(0x14, &reg, &reg);

   return reg.x.ax == 0x55AA;
}

BiosComPort_t *comgetportinfo(int portno)
{
   union REGS           reg;
   struct SREGS         sreg;
   static BiosComPort_t info;

   reg.x.dx = portno;
   reg.h.ah = COMGETPORTINFO;
#if defined (__COMPACT__) || defined (__LARGE__) || defined (__HUGE__)
   reg.x.di = FP_OFF(&info);
   sreg.es = FP_SEG(&info);
#else
   reg.x.di = (int) &info;
   sreg.es = _DS;
#endif
   int86x(0x14, &reg, &reg, &sreg);

   return &info;
}

void comrxoff(int portno)
{
   union REGS reg;

   reg.x.dx = portno;
   reg.h.ah = COMRXOFF;
   int86(0x14, &reg, &reg);
}

void comrxon(int portno)
{
   union REGS reg;

   reg.x.dx = portno;
   reg.h.ah = COMRXON;
   int86(0x14, &reg, &reg);
}

void comsettimeout(int portno, unsigned int timeout)
{
   union REGS reg;

   reg.x.dx = portno;
   reg.h.ah = COMSETTIMEOUT;
   reg.x.cx = timeout;
   int86(0x14, &reg, &reg);
}

unsigned int comgettimeout(int portno)
{
   union REGS reg;

   reg.x.dx = portno;
   reg.h.ah = COMGETTIMEOUT;
   int86(0x14, &reg, &reg);

   return reg.x.cx;
}

int comsetbaud(int portno, unsigned long baud)
{
    union REGS reg;

    reg.x.dx = portno;
    reg.x.bx = baud >> 16;
    reg.x.cx = (unsigned int) baud;
    reg.h.ah = COMSETBAUD;
    int86(0x14, &reg, &reg);

    return reg.x.ax;
}

int comgetbaud(int portno, unsigned long *baud)
{
   union REGS reg;

   reg.x.dx = portno;
   reg.h.ah = COMGETBAUD;
   int86(0x14, &reg, &reg);
   *baud = reg.x.bx;
   *baud = (*baud << 16) + reg.x.cx;

   return reg.x.ax;
}

