/*
 * PCCOM dos ioctl function upon PCCOM ports
 */
#include <io.h>

#include "comlib.h"
#include "comdos.h"

/*
 * constants used in comdos.c
 */
#define	IOCTLIN		0x02
#define IOCTLOUT	0x03

int ioctlstat(int h)
{
   IOCTLIN_t stat;
   int	     sz = sizeof(IOCTLIN_t);

   stat.cbc = IOCTLGETSTAT;
   ioctl(h, IOCTLIN, &stat, &sz);

   return stat.cb.PortSts;
}

char ioctlgetparam(int h)
{
   IOCTLIN_t param;
   int	     sz = sizeof(IOCTLIN_t);

   param.cbc = IOCTLGETPARAM;
   ioctl(h, IOCTLIN, &param, &sz);

   return param.cb.PortParam;
}

void ioctlgetinfo(int h, unsigned int *baud, int *db, int *parity, int *sb)
{
   unsigned char r, n;
   const unsigned int b[8] = {
      B19200, B38400, B300, B600, B1200, B2400, B4800, B9600
   };
   const int d[4] = {
      CS5, CS6, CS7, CS8
   };
   const int s[2] = {
      ST1, ST2
   };
   const int p[4] = {
      PNONE, PODD, PNONE, PEVEN
   };

   r = ioctlgetparam(h);
   n = (r >> 5) & 7;
   *baud = b[n];
   n = (r >> 3) & 3;
   *parity = p[n];
   n = (r >> 2) & 1;
   *sb = s[n];
   n = r & 3;
   *db = d[n];
}

char ioctlgetMCR(int h)
{
   IOCTLIN_t mc;
   int	     sz = sizeof(IOCTLIN_t);

   mc.cbc = IOCTLGETMC;
   ioctl(h, IOCTLIN, &mc, &sz);

   return mc.cb.PortMC;
}

char ioctlgetFC(int h)
{
   IOCTLIN_t fc;
   int	     sz = sizeof(IOCTLIN_t);

   fc.cbc = IOCTLGETFC;
   ioctl(h, IOCTLIN, &fc, &sz);

   return fc.cb.PortFC;
}

int ioctlbufstat(int h)
{
   IOCTLIN_t bufstat;
   int	     sz = sizeof(IOCTLIN_t);

   bufstat.cbc = IOCTLGETDATASZ;
   ioctl(h, IOCTLIN, &bufstat, &sz);

   return bufstat.cb.PortRxDataSz;
}

/*
 *      func :  set communication parameters
 *
 *      input :
 *              portno  : port number   0 .. 64
 *              baud    : baud rate     300, 600, 1200, 2400, 9600, 19200, 38400
 *              db      : data bits     5, 6, 7, 8
 *              parity  : parity bit    0 : nonparity, 1 : odd parity, 3 : even parity
 *              sb      : stop bits     0 : 1 bit, 1 : 2 bits
 *      output :
 *              sts     : port status
 */
int ioctlinit(int h, unsigned int baud, int db, int parity, int sb)
{
   unsigned char cmd = 0x00;
   IOCTLOUT_t    param;
   int	         sz = sizeof(IOCTLOUT_t);

   switch (baud) {
      case B38400 :
	 cmd |= 0x20;
	 break;
      case B19200 :
	 cmd |= 0x00;
	 break;
      case  B9600 :
	 cmd |= 0xE0;
	 break;
      case  B4800 :
	 cmd |= 0xC0;
	 break;
      case  B2400 :
	 cmd |= 0xA0;
	 break;
      case  B1200 :
	 cmd |= 0x80;
	 break;
      case   B600 :
	 cmd |= 0x60;
	 break;
      case   B300 :
	 cmd |= 0x40;
	 break;
      default :
	 return -1;
   }

   switch (parity) {
      case PNONE :              //non parity
	 cmd |= 0x00;
	 break;
      case PODD :               //odd parity
	 cmd |= 0x04;
	 break;
      case PEVEN :              //even parity
	 cmd |= 0x0C;
	 break;
      default :
	 return -1;
   }

   switch (db) {
      case CS5 :
         cmd |= 0x00;
         break;
      case CS6 :
	 cmd |= 0x01;
	 break;
      case CS7 :
	 cmd |= 0x02;
	 break;
      case CS8 :
	 cmd |= 0x03;
	 break;
      default :
	 return -1;
   }

   switch (sb) {
      case ST1 :
         cmd |= 0x00;
         break;
      case ST2 :
         cmd |= 0x04;
         break;
      default :
	 return -1;
   }

   param.cbc = IOCTLSETPARAM;
   param.cb.PortParam = cmd;
   ioctl(h, IOCTLOUT, &param, &sz);

   return 1;
}

void ioctlsetMCR(int h, int mc)
{
   IOCTLOUT_t portmc;
   int	      sz = sizeof(IOCTLOUT_t);

   portmc.cbc = IOCTLSETMC;
   portmc.cb.PortMC = mc;
   ioctl(h, IOCTLOUT, &portmc, &sz);
}

void ioctlsetflow(int h, int fc)
{
   IOCTLOUT_t portfc;
   int	      sz = sizeof(IOCTLOUT_t);

   portfc.cbc = IOCTLSETFC;
   portfc.cb.PortFC = fc;
   ioctl(h, IOCTLOUT, &portfc, &sz);
}

void ioctlbreak(int h, int brk)
{
   IOCTLOUT_t portbrk;
   int	      sz = sizeof(IOCTLOUT_t);

   portbrk.cbc = IOCTLSETBREAK;
   portbrk.cb.PortBreak = brk;
   ioctl(h, IOCTLOUT, &portbrk, &sz);
}

void ioctlpurgebuf(int h)
{
   IOCTLOUT_t purgebuf;
   int	      sz = sizeof(IOCTLOUT_t);

   purgebuf.cbc = IOCTLPURGERXBUF;
   ioctl(h, IOCTLOUT, &purgebuf, &sz);
}